def pad(b: bytes, block_size: int):
    """padding bytes"""
    a = block_size - len(b) % block_size
    return b + a * chr(a).encode()


def unpad(b: bytes):
    """unpadding bytes"""
    return b[: -ord(b[-1:])]



import random

HEXDIGITS = "0123456789abcdefABCDEF"
HEXDIGITS_LOWERCASE = "0123456789abcdef"
HEXDIGITS_UPPERCASE = "0123456789ABCDEF"


def randhex(k: int):
    return "".join(random.choices(HEXDIGITS, k=k))


def randlowerhex(k: int):
    return "".join(random.choices(HEXDIGITS_LOWERCASE, k=k))


def randupperhex(k: int):
    return "".join(random.choices(HEXDIGITS_UPPERCASE, k=k))



from __future__ import annotations

__all__ = [
    # Major class
    "StrDecs",
    "Decimal",
    # Check
    "intable",
    "floatable",
    "is_num",
    # Convert
    "convert_sn",
    "int2str",
    "str2int",
]

import fractions as _fractions
import sys as _sys
from typing import Optional as _Optional

from hmeqotools.mathutil import funcs as mfn


def intable(s: str):
    if s[0] in "-+":
        return s[1:].isdecimal()
    return s.isdecimal()


def floatable(s: str):
    if s[0] in "-+":
        s = s[1:]
    index = s.find(".")
    if index in (-1, 0, len(s) - 1):
        return False
    s = s[:index] + s[index + 1 :]
    return s.isdecimal()


def is_num(s: str):
    return intable(s) or floatable(s)


def convert_sn(str_num: str):
    """Convert scientific notation.
    把科学计数法格式的数字串转换成正常格式的数字串.
    """
    import re

    a = re.search(r"[eE]", str_num)
    if a:
        a = a.start()
        str_num_list = [str_num[:a], int(str_num[a + 1 :])]
        dp = StrDecs(str_num_list[0])
        if str_num_list[1] < 0:
            a = abs(str_num_list[1])
            str_num = dp.floats2ints(str_num_list[0])
            dp.dp += a
            str_num = dp.ints2floats(str_num)
        else:
            a = abs(str_num_list[1]) - dp.dp
            str_num = dp.floats2ints(str_num_list[0])
            if a > 0:
                str_num += "0" * a
                dp.dp = 0
            elif a < 0:
                dp.dp = abs(a)
                str_num = dp.ints2floats(str_num)
    return str_num


if hasattr(_sys, "get_int_max_str_digits"):

    def str2int(obj) -> int:
        if isinstance(obj, str):
            result = 0
            for char in obj:
                result = (result * 10) + ord(char) - 48
            return result
        return int(obj)

    def int2str(__n: int) -> str:
        result = []
        while __n:
            __n, a = divmod(__n, 10)
            result.append(str(a))
        return "".join(reversed(result))

else:
    str2int = int  # type: ignore
    int2str = str


class StrDecs:
    def __init__(self, *ss):
        self.dp = self.get_max_dp(*ss) if ss else 0

    @classmethod
    def get_max_dp(cls, *ss):
        """迭代对象中的字符数字串的最大小数位数"""
        return max(map(cls.get_dp, ss))

    @staticmethod
    def get_dp(s: str):
        """一个字符数字串的最大小数位数"""
        index = s.find(".")
        return 0 if index == -1 else len(s) - 1 - index

    @staticmethod
    def rstrip_zero(s: str):
        """清除末尾多余的零"""
        s = s.rstrip("0")
        return s + "0" if s.endswith(".") else s

    def reserve_dp(self, s: str, dp: int):
        """保留小数, 对传入的字符串数值操作"""
        p = len(s) - self.dp + dp
        s = s.ljust(p, "0")[:p]
        return s[:-1] if s.endswith(".") else s

    def floats2ints(self, s: str):
        """小数字符串转换成整数字符串, 小数部分长度至少大于等于 `self.dp`"""
        if self.dp:
            a = s.find(".")
            if a == -1:
                return s.ljust(len(s) + self.dp, "0")
            else:
                return s.ljust(self.dp + a + 1, "0").replace(".", "", 1)
        return s

    def ints2floats(self, s: str):
        """整数字符串转换回小数字符串."""
        if self.dp:
            s = s.zfill(self.dp + (2 if s[0] in "-+" else 1))
            sl = (s[: -self.dp], s[-self.dp :])
            if int(sl[1]):
                return ".".join(sl)
            return sl[0]
        return s


class Decimal:
    display_precision: _Optional[int] = None

    def __init__(self, int_n=0, decimal_place=0, precision=16):
        for i in (int_n, decimal_place, precision):
            if not isinstance(i, int):
                raise TypeError("Type must be %s: %r" % (int.__name__, i))
        for i in (decimal_place, precision):
            if i < 0:
                raise ValueError("Can not less than 0: %r" % i)
        # 当前小数位数
        self.dp = decimal_place
        # 数值
        self.n = int_n
        # 1
        self.one = 10**self.dp
        if precision:
            self.magnify_precision(precision)

    def __repr__(self):
        # 保留 n 位数字
        n = 16
        ss = self.__str__()[: n + 3]
        sign = "" if ss[0].isdecimal() else ss[0]
        # 数字和小数点
        d = ss[1:] if sign else ss
        # 数字个数大于 n
        if len(d.replace(".", "", 1)) > n:
            if "." in d:
                d = d[: n + 1]
                # 如果小数点在末尾，省去小数点
                if d[-1] == ".":
                    d = d[:n]
            else:
                d = d[:n]
            d += "..."
        return "{}('{}{}')".format(self.__class__.__name__, sign, d)
        # return "{}('{}')".format(self.__class__.__name__, self.__str__())

    def __str__(self):
        integer, decimals = self.separate()
        if self.display_precision is not None:
            decimals = decimals[: self.display_precision]
        if self.n < 0:
            integer = "-" + integer
        if any(i != "0" for i in decimals):
            return ".".join([integer, decimals])
        return integer

    def __int__(self):
        return self.n // self.one

    def __float__(self):
        # result = float(".".join(self.separate()))
        # return -result if self._n < 0 else result
        return self.n / self.one

    def __complex__(self):
        return complex(str(self))

    def __bool__(self):
        return bool(self.n)

    def __abs__(self):
        new = self.copy()
        new.n = abs(new.n)
        return new

    def __pos__(self):
        return self.copy()

    def __neg__(self):
        new = self.copy()
        new.n = -new.n
        return new

    def __round__(self, n=None):
        return round(float(self), n)

    def __divmod__(self, other):
        div = self // other
        return div, self - div * other

    def __add__(self, other):
        if isinstance(other, self.__class__):
            new = self
            if new.dp < other.dp:
                new, other = other, new
            new = new.copy()
            new.n += new.magnify_int(other.n, new.dp - other.dp)
            return new
        other_type = type(other)
        if other_type is int or other_type is bool:
            new = self.copy()
            new.n += new.one * other
            return new
        elif other_type is float:
            return self + self.convert_float(other)
        raise TypeError()

    def __sub__(self, other):
        if isinstance(other, self.__class__):
            new = self.merge(other)
            new.n -= new.magnify_int(other.n, new.dp - other.dp)
            return new
        other_type = type(other)
        if other_type is int or other_type is bool:
            new = self.copy()
            new.n -= new.one * other
            return new
        elif other_type is float:
            return self - self.convert_float(other)
        raise TypeError()

    def __mul__(self, other):
        if isinstance(other, self.__class__):
            new = self
            if new.dp < other.dp:
                new, other = other, new
            new = new.copy()
            new.n = new.n * other.n // other.one
            return new
        other_type = type(other)
        if other_type is int or other_type is bool:
            new = self.copy()
            new.n *= other
            return new
        elif other_type is float:
            # 乘法特殊，不需要小数位数相同
            # 所以此处使用 self.from_float 而不是 self.convert_float
            other = self.from_float(other)
            # 保证小数位数不受浮点数影响
            other.reduce_precision(self.dp)
            return self * other
        raise TypeError()

    def __truediv__(self, other):
        if isinstance(other, self.__class__):
            new = self.merge(other)
            new.n *= new.one
            new.n //= new.magnify_int(other.n, new.dp - other.dp)
            return new
        other_type = type(other)
        if other_type is int or other_type is bool:
            new = self.copy()
            new.n //= other
            return new
        elif other_type is float:
            return self / self.convert_float(other)
        raise TypeError()

    def __floordiv__(self, other):
        if isinstance(other, self.__class__):
            new = self.merge(other)
            new.n //= new.magnify_int(other.n, new.dp - other.dp)
            new.one = 1
            new.dp = 0
            return new
        other_type = type(other)
        if other_type is int or other_type is bool:
            new = self.copy()
            new.n = new.n // new.one // other
            new.one = 1
            new.dp = 0
            return new
        elif other_type is float:
            return self // self.convert_float(other)
        raise TypeError()

    def __mod__(self, other):
        return self - (self // other) * other

    def __pow__(self, power, modulus=None):
        if isinstance(power, self.__class__):
            new = self.merge(power)
            if power != 0:
                # 分离指数的整数和小数部分
                p_int, p_dec = power.separate()
                p_int = -int(p_int) if power < 0 else int(p_int)
                # 将小数部分转换成分数，用于开方
                p_dec = "-0." + p_dec if p_int < 0 else "0." + p_dec
                p_dec = _fractions.Fraction(p_dec)
                # 如果底数是负数，指数是小数
                if new < 0 and p_dec:
                    return complex(new) ** complex(p_dec)
                n = new.copy()
                # 计算整数部分
                if p_int:
                    n **= p_int
                # 计算小数部分
                if p_dec:
                    dms = new.root(p_dec.denominator) ** p_dec.numerator
                    # 整数部分不为 0
                    if p_int:
                        n *= dms
                    # 整数部分为 0 时，只保留小数部分计算结果
                    else:
                        n = dms
                new = n
            # power 为 0
            else:
                new.n = new.one
            return new if modulus is None else new % modulus
        power_type = type(power)
        if power_type is int or power_type is bool:
            new = self.copy()
            if power != 0:
                # 指数为负数，求底数的倒数，将指数转换成正数
                if power < 0:
                    new = 1 / new
                    power = abs(power)
                if power > 1:
                    new.n = new.n**power // new.one ** (power - 1)
            else:
                new.n = new.one
            return new if modulus is None else new % modulus
        elif power_type is float:
            return self ** self.from_float(power)
        raise TypeError()

    def __radd__(self, other):
        other_type = type(other)
        if other_type is int or other_type is bool:
            new = self.copy()
            new.n = new.one * other + new.n
            return new
        elif other_type is float:
            return self.convert_float(other) + self
        raise TypeError()

    def __rsub__(self, other):
        other_type = type(other)
        if other_type is int or other_type is float:
            new = self.copy()
            new.n = new.one * other - new.n
            return new
        elif other_type is float:
            return self.convert_float(other) - self
        raise TypeError()

    def __rmul__(self, other):
        other_type = type(other)
        if other_type is int or other_type is bool:
            new = self.copy()
            new.n = other * new.n
            return new
        elif other_type is float:
            other = self.from_float(other)
            other.reduce_precision(self.dp)
            return self * other
        raise TypeError()

    def __rtruediv__(self, other):
        other_type = type(other)
        if other_type is int or other_type is bool:
            new = self.copy()
            new.n = new.one**2 * other // new.n
            return new
        elif other_type is float:
            return self.convert_float(other) / self
        raise TypeError()

    def __rfloordiv__(self, other):
        other_type = type(other)
        if other_type is int or other_type is bool:
            new = self.copy()
            new.n = new.one * other // new.n
            new.one = 1
            new.dp = 0
            return new
        elif other_type is float:
            return self.convert_float(other) // self
        raise TypeError()

    def __rmod__(self, other):
        return other - (other // self) * self

    def __rpow__(self, other):
        other_type = type(other)
        if other_type is int or other_type is bool:
            return self.convert_int(other) ** self
        elif other_type is float:
            return self.convert_float(other) ** self
        raise TypeError()

    def __eq__(self, other):
        if isinstance(other, self.__class__):
            return self.merge(other).n == other.merge(self).n
        other_type = type(other)
        if other_type is int or other_type is bool:
            return self.n == self.one * other
        elif other_type is float:
            return self.n == self.convert_float(other).n
        return False

    def __ge__(self, other):
        if isinstance(other, self.__class__):
            return self.merge(other).n >= other.merge(self).n
        other_type = type(other)
        if other_type is int or other_type is bool:
            return self.n >= self.one * other
        elif other_type is float:
            return self.n >= self.convert_float(other).n
        return False

    def __le__(self, other):
        if isinstance(other, self.__class__):
            return self.merge(other).n <= other.merge(self).n
        other_type = type(other)
        if other_type is int or other_type is bool:
            return self.n <= self.one * other
        elif other_type is float:
            return self.n <= self.convert_float(other).n
        return False

    def __gt__(self, other):
        if isinstance(other, self.__class__):
            return self.merge(other).n > other.merge(self).n
        other_type = type(other)
        if other_type is int or other_type is bool:
            return self.n > self.one * other
        elif other_type is float:
            return self.n > self.convert_float(other).n
        return False

    def __lt__(self, other):
        if isinstance(other, self.__class__):
            return self.merge(other).n < other.merge(self).n
        other_type = type(other)
        if other_type is int or other_type is bool:
            return self.n < self.one * other
        elif other_type is float:
            return self.n < self.convert_float(other).n
        return False

    @staticmethod
    def magnify_int(num, dp):
        if dp > 0 and num:
            return num * 10**dp
        return num

    @staticmethod
    def reduce_int(num, dp):
        if dp > 0 and num:
            return num // 10**dp
        return num

    def magnify_precision(self, dp):
        """放大整数，增加小数位数.
        如果新的小数位数小于原本，则什么都不做.
        """
        dp -= self.dp
        if dp > 0:
            self.dp += dp
            n = 10**dp
            if self.n:
                self.n *= n
            self.one *= n
        return None

    def reduce_precision(self, dp):
        """缩小整数，减少小数位数.
        如果新的小数位数大于原本，则什么都不做.
        """
        dp = self.dp - dp
        if dp > 0:
            self.dp -= dp
            n = 10**dp
            if self.n:
                self.n //= n
            self.one //= n
        return None

    def change_precision(self, dp):
        if self.dp < dp:
            self.magnify_precision(dp)
        elif self.dp > dp:
            self.reduce_precision(dp)
        return None

    def merge(self, obj):
        """将与另一个此类型对象合并成一个新对象."""
        new = self.copy()
        if obj.dp > new.dp:
            new.n *= 10 ** (obj.dp - new.dp)
            new.one = obj.one
            new.dp = obj.dp
        return new

    @classmethod
    def from_int(cls, n, precision=0):
        """将整数类型转换成此类型."""
        return cls(n, 0, precision)

    @classmethod
    def from_float(cls, n, precision=0):
        """将浮点类型转换成此类型."""
        return cls.from_str(str(n), precision)

    @classmethod
    def from_str(cls, n, precision=0):
        """将数字串转换成此类型."""
        n = convert_sn(n)
        dp = StrDecs(n)
        return cls(int(dp.floats2ints(n)), dp.dp, precision)

    def convert_int(self, n):
        """以自身为模板，将整数类型转换成此类型."""
        new = self.copy()
        new.n = n * new.one
        return new

    def convert_float(self, n):
        """以自身为模板，将浮点类型转换成此类型."""
        return self.convert_str(str(n))

    def convert_str(self, n):
        """以自身为模板，将数字串转换成此类型."""
        new = self.copy()
        n = convert_sn(n)
        dp = StrDecs(n)
        n = int(dp.floats2ints(n))
        new.n = new.one * n // 10**dp.dp
        return new

    def root(self, exponent=2) -> Decimal:
        """开根号."""
        if isinstance(exponent, float):
            exponent = self.from_float(exponent)
        if exponent == 0:
            new = self.copy()
            new.n = new.one
            return new
        elif exponent == 1:
            return self.copy()
        return mfn.root(self, exponent)

    def separate(self):
        """分割整数和小数部分."""
        # integer = str(abs(self._n)).zfill(self.dp + 1)
        # return integer[:-self.dp], integer[-self.dp:]
        integer, decimals = divmod(abs(self.n), self.one)
        return int2str(integer), int2str(decimals).zfill(self.dp)

    def copy(self):
        new = self.__class__()
        new.__dict__ = self.__dict__.copy()
        return new

    def get_n(self):
        """获取内部存储的整数."""
        return self.n

    def get_one(self):
        return self.one

    def get_decimal_place(self):
        """获取小数位数."""
        return len(StrDecs.rstrip_zero(self.separate()[1]))

    def get_precision(self):
        """获取精度."""
        return self.dp

    def get_full(self):
        result = ".".join(self.separate())
        if self.n < 0:
            result = "-" + result
        return result


def main():
    n = Decimal.from_str("2", 5000)
    n.display_precision = 16
    a = n + 123.45
    b = n.root(2)
    print(2 + 123.45, 2**0.5)
    print(a, b)
    print(a.get_full())
    print(b.get_full())


if __name__ == "__main__":
    main()



def predict_digits(bit_digits: int) -> int:
    """预测二进制长度对应的十进制长度"""
    w = 0.3010281762263712
    b = 0.5015807678542549
    y = w * bit_digits + b
    return round(y)


def main():
    a = []
    for i in range(1, 20000 + 1):
        pre = predict_digits(i) == len(str(2**i))
        # print(i, pre)
        a.append(pre)
    print("失误次数:", a.count(False))


if __name__ == "__main__":
    main()



"""
Usage:

```
@require_lock
def test():
    global n
    for _ in range(1000000):
        n += 1

@require_lock
def test2():
    test()

n = 0

thr1 = threading.Thread(target=test)
thr2 = threading.Thread(target=test2)
thr1.start()
thr2.start()
thr1.join()
thr2.join()

print(n)
```
"""

from __future__ import annotations

import threading
from typing import Any, Callable, Literal, Protocol, TypeVar, overload

GLOBAL_LOCK = threading.RLock()

AnyFunc = TypeVar("AnyFunc", bound=Callable)


class LockProtocol(Protocol):
    def acquire(self, blocking=True, timeout=-1) -> bool | Literal[1]: ...

    def release(self) -> None: ...


@overload
def lock_required(func: AnyFunc, lock: LockProtocol = GLOBAL_LOCK) -> AnyFunc: ...


@overload
def lock_required(func=None, lock: LockProtocol = GLOBAL_LOCK) -> Callable[[AnyFunc], AnyFunc]: ...


def lock_required(
    func: AnyFunc | None = None, lock: LockProtocol = GLOBAL_LOCK
) -> AnyFunc | Callable[[AnyFunc], AnyFunc]:
    if func is None:

        def get_func(__func: AnyFunc):
            return lock_required(__func)

        return get_func

    def wrapper(*args, **kwds) -> Any:
        lock.acquire()
        try:
            result = func(*args, **kwds)
        except Exception:
            result = None
        finally:
            lock.release()
        return result

    wrapper.__name__ = func.__name__
    return wrapper



from __future__ import annotations

import queue
import threading
import traceback
from concurrent.futures import ThreadPoolExecutor
from typing import Any, Callable, Iterable, TypeVar, overload


def on_error(exc: Exception):
    print(
        "--------------------------------------------------\n"
        + "".join(traceback.format_exception(type(exc), exc, exc.__traceback__))
        + "\n--------------------------------------------------"
    )


class Task:
    """Basic task."""

    event_type = threading.Event

    def __init__(self, target: Callable | None = None, *, args: Iterable = (), kwds: dict | None = None):
        self.target = target
        self.args = args
        self.kwargs = kwds if kwds else {}
        self.result = None
        self.exc = None
        self.finished = self.event_type()

    def __call__(self):
        try:
            self.result = self.run()
        except Exception as exc:
            self.exc = exc
            on_error(exc)
        finally:
            self.finished.set()

    def run(self) -> Any:
        if self.target is None:
            return None
        return self.target(*self.args, **self.kwargs)

    def wait(self, timeout: float | None = None):
        return self.finished.wait(timeout=timeout)

    def set_default(self, default):
        self.result = default

    @property
    def noerror(self):
        """Not error."""
        return self.exc is None


class TaskRunner:
    """The threading safe unified execution.

    Usage:
    ```
    with TaskRunner() as tr:
        tr.submit(print, args=("Ln1: Hello World",))
        tr.put(lambda: print("Ln1: Hello World"))
    ```

    ```
    tr = TaskRunner()
    @tr.putter
    def test():
        pass
    test()
    tr.close()
    tr.join()
    ```
    """

    def __init__(self):
        self.queue: queue.Queue[Callable | None] = queue.Queue()
        self.thr = threading.Thread(target=self.mainloop, daemon=True)
        self.started = False
        self.alive = threading.Event()
        self.ended = threading.Event()
        self.ended.set()

    def __enter__(self):
        if not self.started:
            self.start()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.close()
        self.ended.wait()

    def run(self):
        while self.alive.is_set():
            task = self.queue.get()
            if task is None:
                self.alive.clear()
                break
            task()

    def mainloop(self):
        self.run()
        self.ended.set()

    def start(self):
        """Start the threading."""
        self.started = True
        self.ended.clear()
        self.alive.set()
        self.thr.start()

    def close(self):
        """TaskRunner will closed after all task terminated."""
        self.queue.put(None)

    def stop(self):
        """TaskRunner will closed after current task terminated."""
        self.alive.clear()
        self.close()

    def join(self):
        """Wait the TaskRunner terminated."""
        self.ended.wait()

    def put(self, func: Callable[[], Any]):
        """Put a callable to queue."""

        def wrapper():
            try:
                func()
            except Exception as exc:
                on_error(exc)

        self.queue.put(wrapper)

    def putter(self, func: Callable):
        """A decorator of `.put`."""
        return lambda *a, **kwa: self.put(lambda: func(*a, **kwa))

    def submit(self, func: Callable, args: Iterable = (), kwds: dict | None = None, *, default=None, handler=Task):
        """Put a Task to queue."""
        task = handler(func, args=args, kwds=kwds)
        if default is not None:
            task.result = default
        self.queue.put(task)
        return task

    Handler = TypeVar("Handler", bound=Task)

    @overload
    def submitter(
        self, *, default=None, handler: type[Handler] = Task
    ) -> Callable[[Callable], Callable[..., Handler]]: ...

    @overload
    def submitter(self, func: Callable, *, default=None, handler: type[Handler] = Task) -> Callable[..., Handler]: ...

    def submitter(self, func=None, *, default=None, handler=Task) -> Any:
        """A decorator of `.run`."""
        if func is None:
            return lambda f: self.submitter(f, default=default, handler=handler)
        return lambda *a, **kwa: self.submit(func, a, kwa, default=default, handler=handler)

    def proxy(self, *, default=None, handler=Task):
        def get_func(func: Callable):
            def wrapper(*args, **kwds):
                task = self.submit(func, args=args, kwds=kwds, default=default, handler=handler)
                task.wait()
                return task.result

            return wrapper

        return get_func


class ThreadTaskRunner(TaskRunner):
    def __init__(self):
        super().__init__()
        self.thread_count = 4

    def run(self):
        with ThreadPoolExecutor(self.thread_count) as pool:
            while self.alive.is_set():
                task = self.queue.get()
                if task is None:
                    self.alive.clear()
                    break
                pool.submit(task)



from __future__ import annotations

import os
import sys


def is_writable(path: str):
    """Test if the path is writable."""
    try:
        os.chmod(path, os.stat(path).st_mode)
    except OSError:
        return False
    return True


def get_profile_path() -> str:
    return os.path.expanduser("~")


def get_desktop_path() -> str:
    return os.path.expanduser("~/Desktop")


if sys.platform == "win32":

    def get_mounts() -> list[str]:
        """Get the mount points (currently only supported on Windows)."""
        result = []
        for name in os.popen("wmic logicaldisk get Name"):
            name = name.strip()
            if name and not name.endswith(os.sep) and os.path.exists(name):
                result.append(name)
        return result


def main():
    print(get_desktop_path())


if __name__ == "__main__":
    main()



__all__ = [
    # Round
    "ceil",
    "floor",
    # Power, modulus
    "pow",
    "powmod",
    # Root
    "root",
    "enrt",
    "newton_method",
    "dichotomy_root",
    # Factorial
    "factorial",
    "factorial_gen",
    "factorial_stirling",
    # Prime
    "prime",
    "is_prime",
    "primes",
    "prime_miller_rabin",
    "prime_factorization",
    # Formulas, Functions
    "gcd",
    "gcd_ext_euclid",
    "sin",
    "cos",
    "tan",
    "asin",
    "acos",
    "atan",
    "sin2",
    "cos2",
    "fib",
    "fib_nth",
    # Constants
    "pi",
    "e",
    "dec_pi",
    "dec_e",
    "dec_2",
    "prime_list_k",
    # formulas
    "binomial_theorem",
]

import decimal
import math
import random
from typing import Any, Union, overload

pi = math.pi
e = math.e

dec_pi = decimal.Decimal.from_float(pi)
dec_e = decimal.Decimal.from_float(e)
dec_2 = decimal.Decimal(2)

# 一千以内所有素数
prime_list_k = (
    2,
    3,
    5,
    7,
    11,
    13,
    17,
    19,
    23,
    29,
    31,
    37,
    41,
    43,
    47,
    53,
    59,
    61,
    67,
    71,
    73,
    79,
    83,
    89,
    97,
    101,
    103,
    107,
    109,
    113,
    127,
    131,
    137,
    139,
    149,
    151,
    157,
    163,
    167,
    173,
    179,
    181,
    191,
    193,
    197,
    199,
    211,
    223,
    227,
    229,
    233,
    239,
    241,
    251,
    257,
    263,
    269,
    271,
    277,
    281,
    283,
    293,
    307,
    311,
    313,
    317,
    331,
    337,
    347,
    349,
    353,
    359,
    367,
    373,
    379,
    383,
    389,
    397,
    401,
    409,
    419,
    421,
    431,
    433,
    439,
    443,
    449,
    457,
    461,
    463,
    467,
    479,
    487,
    491,
    499,
    503,
    509,
    521,
    523,
    541,
    547,
    557,
    563,
    569,
    571,
    577,
    587,
    593,
    599,
    601,
    607,
    613,
    617,
    619,
    631,
    641,
    643,
    647,
    653,
    659,
    661,
    673,
    677,
    683,
    691,
    701,
    709,
    719,
    727,
    733,
    739,
    743,
    751,
    757,
    761,
    769,
    773,
    787,
    797,
    809,
    811,
    821,
    823,
    827,
    829,
    839,
    853,
    857,
    859,
    863,
    877,
    881,
    883,
    887,
    907,
    911,
    919,
    929,
    937,
    941,
    947,
    953,
    967,
    971,
    977,
    983,
    991,
    997,
)

prime_set_k = set(prime_list_k)


def ceil(num):
    """向上取整"""
    result = int(num)
    if result == num:
        return result
    return result + 1


def floor(num):
    """向下取整"""
    if isinstance(num, int):
        return num
    return int(num // 1)


def pow(n, e: int):
    result = 1
    while e:
        if e & 1:
            result *= n
        n *= n
        e >>= 1
    return result


@overload
def powmod(n: int, e: int, m: int) -> int: ...


@overload
def powmod(n: float, e: int, m: float) -> float: ...


def powmod(n, e, m):
    """幂取模"""
    n %= m
    result = 1
    while e:
        if e & 1:
            result = result * n % m
        n = n * n % m
        e >>= 1
    return result


def enrt(func, n, exponent: Union[int, float] = 2, *args, **kwargs):
    """丰富根号计算结果"""
    # 指数为0返回1
    if exponent == 0:
        return 1
    # 指数为1返回这个数本身
    elif exponent == 1:
        return n

    # 底数指数是否是负数
    n_is_neg = n < 0
    exponent_is_neg = exponent < 0

    # 如果底数是负数
    if n_is_neg:
        # 如果指数是小数或是偶数，则涉及复数计算
        if isinstance(exponent, float) or not exponent & 1:
            return complex(n) ** (1 / complex(exponent))
        # 先取正数，最后再改回负数
        else:
            n = abs(n)
    # 如果指数是负数，n取倒数，指数取正数
    if exponent_is_neg:
        n = 1 / n
        exponent = abs(exponent)

    # 计算
    result = func(n, exponent, *args, **kwargs)

    # 如果n原本是负数, 取负数
    if n_is_neg:
        result = -result
    return result


def newton_method(n, exponent: Any = 2):
    """牛顿迭代法"""
    x0 = 1
    e0 = exponent - 1
    _a = x0
    x_n = x0 + (n / x0**e0 - x0) / exponent
    while _a != x_n:
        _a = x0
        x0 = x_n
        x_n = x0 + (n / x0**e0 - x0) / exponent if x0 else x0
    return x_n


def dichotomy_root(n, exponent: Any = 2):
    """二分法"""
    low = 0
    high = n
    a = n
    result = 0
    while result != a:
        a = result
        result = (low + high) / 2
        b = result**exponent
        if b > n:
            high = result
        elif b < n:
            low = result
    return result


def factorial(n: int):
    """阶乘"""
    result = 1
    # 计算 n 整除和取余 2 的值
    # 如果没有余数, result 乘以 n 减一的双阶乘
    # 如果有余数, result 乘以 n 的双阶乘
    # 将 n 整除 2 的结果赋值给 n
    # result 乘以 2 的 n 次方
    # 不断循环以上步骤, 直到 n 等于 1
    pw2 = 0
    while n > 1:
        a = divmod(n, 2)
        result *= double_factorial(n if a[1] else n - 1)
        n = a[0]
        pw2 += n
    result <<= pw2
    return result


def double_factorial(n: int):
    """双阶乘"""
    result = 1
    for i in range(n, 1, -2):
        result *= i
    return result


def factorial_gen(n: int):
    yield 1
    result = 1
    for i in range(1, n + 1):
        result *= i
        yield result


def factorial_stirling(n):
    """斯特林公式.
    斯特林公式: 取 n 的阶乘的近似值的数学公式.
    """
    n = decimal.Decimal(str(n))
    return round(root(dec_2 * dec_pi * n) * (n / dec_e) ** n)


def prime(n: int, stop=None):
    """n 是否是质数，可指定返回范围内的质数 n ~ stop(不包含stop)."""
    if not stop:
        return is_prime(n)
    result = [2] if n <= 2 else []
    # 偶数加一
    if not n & 1:
        n += 1
    # 此部分负责小于1001的数
    start_index = None
    end_index = None
    for index, i in enumerate(prime_list_k):
        if start_index is None:
            if i >= n:
                start_index = index
        elif i >= stop:
            end_index = index
            break
    if start_index:
        result.extend(prime_list_k[start_index:end_index])
    # 此部分负责大于等于1001的数
    if stop > 1001:
        for x in range(1001, stop, 2):
            if prime_miller_rabin(x):
                result.append(x)
    return result


def is_prime(n: int):
    if not n & 1:
        return False
    if n <= 1000:
        return n in prime_set_k
    for i in prime_list_k:
        if not n % i:
            return False
    return prime_miller_rabin(n)


def primes(n: int):
    """Returns a list of primes < n"""
    half_n = n // 2
    sieve = [True] * half_n
    for i in range(3, int(n**0.5) + 1, 2):
        if sieve[i // 2]:
            sieve[i * i // 2 :: i] = [False] * ((n - i * i - 1) // (2 * i) + 1)
    return [2] + [2 * i + 1 for i in range(1, half_n) if sieve[i]]


def prime_miller_rabin(num: int):
    """判断一个数是否是质数，这个数不能太小."""
    s = num - 1
    t = 0
    while s & 1 == 0:
        s = s // 2
        t += 1

    for _ in range(5):
        a = random.randrange(2, num - 1)
        v = powmod(a, s, num)
        if v != 1:
            i = 0
            while v != num - 1:
                if i == t - 1:
                    return False
                else:
                    i = i + 1
                    v = v**2 % num
    return True


def prime_factorization(num: int):
    """质因数分解."""
    result = []
    while not num % 2:
        num //= 2
        result.append(2)
    i = 3
    while i <= num:
        if num % i:
            i += 2
            continue
        num //= i
        result.append(i)
    return result


def gcd(a, b):
    """最大公约数"""
    while a != 0:
        a, b = b % a, a
    return b


def gcd_ext_euclid(a, b):
    """最大公约数.
    扩展欧里几得算法.
    ax + by = gcd(a, b).
    返回 x, y, q.(q为最大公约数)
    """
    # 递
    lst = [(a, b)]
    a, b = lst[0]
    while True:
        if b == 0:
            break
        else:
            a, b = b, a % b
            lst.append((a, b))
    # 归
    del lst[-1]
    x, y, q = 1, 0, a
    while lst:
        a, b = lst.pop()
        x, y = y, x - (a // b) * y
    return x, y, q


def sin(angle):
    return math.sin(angle * pi / 180)


def cos(angle):
    return math.cos(angle * pi / 180)


def tan(angle):
    return math.tan(angle * pi / 180)


def asin(ratio):
    return math.asin(ratio) * 180 / pi


def acos(ratio):
    return math.acos(ratio) * 180 / pi


def atan(ratio):
    return math.atan(ratio) * 180 / pi


def sin2(x, angle=False):
    # sin x = x - x^3/3! + x^5/5! - x^7/7! + ... + (-1)^n-1 x^(2n-1)/(2n-1)!
    if angle:
        x = x * pi / 180
    result = x
    sign = 1
    x_e = x
    j = 1
    f = 1
    i = 1
    while f:
        i += 2
        sign *= -1
        x_e *= x * x
        j *= i * (i - 1)
        f = sign * x_e / j
        result += f
    return result


def cos2(x, angle=False):
    if angle:
        x = x * pi / 180
    result = pi / 2 - x
    sign = 1
    x_e = x
    j = 1
    f = 1
    i = 1
    while f:
        i += 2
        sign *= -1
        x_e *= x * x
        j *= i * (i - 1)
        f = sign * x_e / j
        result += f
    return result


def fib(n: int):
    a, b = 0, 1
    while a < n:
        yield a
        a, b = b, a + b


def fib_nth(n: int):
    if n >= 2:
        a, b = 0, 1
        yield a
        for _ in range(1, n):
            a, b = b, a + b
            yield a
    elif n == 1:
        yield 1
    elif n == 0:
        yield 0


def binomial_theorem(x, y, n=2):
    """二项式定理 $(x+y)^n$"""
    fac_n = factorial(n)
    return sum(
        fac_n / (k_fac * factorial(n - k)) * x ** (n - k) * y ** (k)
        for k_fac, k in zip(factorial_gen(n + 1), range(n + 1))
    )


root = newton_method



"""Find and delete pycache directory."""

import os
import shutil
from collections import deque

cache_dirnames = ["__pycache__", ".pdm-build"]


def main():
    path_list = deque(["."])
    while path_list:
        for i in os.scandir(path_list.popleft()):
            if not i.is_dir():
                continue
            if i.name in cache_dirnames:
                shutil.rmtree(i.path)
            else:
                path_list.append(i.path)


if __name__ == "__main__":
    main()



from __future__ import annotations

import asyncio
import threading
from typing import Coroutine


class AsyncTaskCenter:
    def __init__(self, event_loop: asyncio.AbstractEventLoop | None = None):
        self.loop = event_loop or asyncio.new_event_loop()

    def run(self):
        self.loop.run_forever()

    def start(self):
        threading.Thread(target=self.loop.run_forever).start()

    def close(self):
        self.put(self.loop.stop)

    def put(self, task):
        if not isinstance(task, Coroutine):

            async def make_task(__task):
                __task()

            task = make_task(task)
        asyncio.run_coroutine_threadsafe(task, self.loop)



import re as _re


class Re:
    """匹配数字."""

    # 符号
    signed = r"[\\+-]?"
    # 无符号整数
    unsigned_int = r"\d+"
    # 整数
    integer = signed + unsigned_int
    # 无符号小数
    unsigned_dec = unsigned_int + r"(\.\d+)"
    # 小数
    decimals = signed + unsigned_dec
    # 无符号数
    unsigned_num = unsigned_dec + r"?"
    # 数
    number = signed + unsigned_num
    # 零
    zero = signed + r"0+(\.0+)?"

    # 编译上面的正则
    signed = _re.compile(signed)
    unsigned_int = _re.compile(unsigned_int)
    integer = _re.compile(integer)
    unsigned_dec = _re.compile(unsigned_dec)
    decimals = _re.compile(decimals)
    unsigned_num = _re.compile(unsigned_num)
    number = _re.compile(number)
    zero = _re.compile(zero)

    # 方法会用到的pattern
    _is_int = _re.compile(r"^" + integer.pattern + r"$")
    _is_float = _re.compile(r"^" + decimals.pattern + r"$")
    _is_num = _re.compile(r"^" + number.pattern + r"$")
    _is_zero = _re.compile(r"^" + zero.pattern + r"$")

    @classmethod
    def is_int(cls, s):
        """是否是整数串."""
        return bool(cls._is_int.match(s))

    @classmethod
    def is_float(cls, s):
        """是否是小数串."""
        return bool(cls._is_float.match(s))

    @classmethod
    def is_num(cls, s):
        """是否是数字串."""
        return bool(cls._is_num.match(s))

    @classmethod
    def is_zero(cls, s):
        """是否为0."""
        return bool(cls._is_zero.match(s))
